#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os
import datetime
import logging

from models import Model

__author__="Balzano Alessandro, aka delta aka alfateam aka alfateam123"
__license__="Creative Commons V3.0 BY-SA-NC"



#-----------------
#impostazioni per logging
LOG_FILENAME='PyPMMLogController.txt'
logging.basicConfig(filename=LOG_FILENAME, level=logging.DEBUG)
#-----------------

class Controller(object):
    """
    The Controller class is an interface used by View
    to access data of Model.
    """
    def __init__(self):
        self.model=Model()
        self.VALUE=self.model.VALUE
        self.CAUSAL=self.model.CAUSAL
        self.NOTES=self.model.NOTES
        self.DIRECTION=self.model.DIRECTION
        self.DAY=self.model.DAY
        self.MONTH=self.model.MONTH
        self.YEAR=self.model.YEAR
        self.EXPENSE_PATH=os.path.join(os.path.dirname(sys.argv[0]), "expenseCausals.txt")
        self.INCOME_PATH=os.path.join(os.path.dirname(sys.argv[0]), "incomeCausals.txt")

    def control_direction(self, direction):
        """ control of 'direction' property """
        if not direction in (u'Expense', u'Income'):
            logging.debug('Controller: direction isn\'t Expense or Income')
            return False
        return True

    def control_date_by_datetime(self, date):
        """ this method is util to control if a date is correct """
        try:
            if datetime.date.today()-date<datetime.timedelta(0):
                logging.debug("Controller: the passed day is in the future...")
                passed_test=False
            else:
                passed_test=True
        except:
            #error: this object is not valid!
            logging.debug("Controller: the passed 'date' parameter type is not valid")
            passed_test=False

        return passed_test

    def is_date_correct(self, date):
        """control of 'date' property"""
        import re
        passed_test=False
        try:
            """
            About the regex used there:
            I noticed both strings "12/12/1993" and "12/12/1993/33/55" are
             tested successfully, and it's not acceptable. So I added ^ and $
             tokens, that are used to indicate begin and end of a string.
            So, "12/12/1993" now passes the test, but "12/12/1993/33/55"
             doesn't.
            """
            pattern=re.compile("^\d{1,2}/\d{1,2}/\d{4}$")
            if re.match(pattern, date):
                day, month, year=(int(item) for item in date.split('/'))
                passed_test=self.control_date_by_datetime(datetime.date(year, month, day))
            else:
                passed_test=False
        except TypeError:
            passed_test=self.control_date_by_datetime(date)

        return passed_test

    def get_date(self, date):
        try:
            #it only verify if date is a string (not an iterable) without modify the parameter.
            x=date.upper()
            value=date
        except TypeError:
            #'date' is certainly a datetime.date object
            day=date.day()
            month=date.month()
            year=date.year()
            value="/".join([day, month, year])

        return value

    def is_causal_correct(self, passed_causal):
        """
        this function controls if the causal is valid.
        All valid causals are listed in two TXT files, "expenseCausals.txt"
         and "incomeCausals.txt"
        this function returns True if causal is valid, else False.
        """
##        during testing, I encountered a problem:
##         a causal (in a file) was followed by a blank char.
##         (Example: u"Foo " (causal into file) != u"Foo" (causal to match))
##        When the function matched this causal, the test failed.
##        I added strip(" ") to fix this bug.

        for path in (self.EXPENSE_PATH, self.INCOME_PATH):
            for causal in open(path):
                if (causal.splitlines()[0]).strip(" ")==passed_causal:
                    return True
        return False


    def is_movement_correct(self, value, causal, notes, direction, date):
        """
        this function return the validity of a movement.
        True if valid, False if not.
        """
        from types import FloatType, IntType, UnicodeType
        is_value_correct=type(value) in [FloatType, IntType]
        is_causal_correct=self.is_causal_correct(causal)
        is_notes_correct=type(notes) is UnicodeType
        is_direction_correct=self.control_direction(direction)
        is_date_correct=self.is_date_correct(date)

        return (is_value_correct and is_causal_correct and is_notes_correct and
                is_direction_correct and is_date_correct)

    def add_movement(self, value, causal, notes, direction="Expense", date=datetime.date.today()):
        """
        this function adds a movement with the passed parameters.
        raises a ValueError if parameters are uncorrect.
        """
        if self.is_movement_correct(value, causal, notes, direction, date):
            self.model.add_movement(value, causal, notes, direction, date)
        else:
            raise ValueError("passed parameters are uncorrect.")

    def update_movement(self, ID, newValuesDict):
        """
        this function updates a movement with the passed parameters.
        raises a ValueError if parameters are uncorrect.
        """
        value=newValuesDict['value']
        causal=newValuesDict['causal']
        notes=newValuesDict['notes']
        direction=newValuesDict['direction']
        day=newValuesDict['day']
        month=newValuesDict['month']
        year=newValuesDict['year']
        date="/".join([str(item) for item in (day, month, year)])
        
        if self.is_movement_correct(value, causal, notes, direction, date):
            self.model.update_movement(ID, newValuesDict)
        else:
            raise ValueError("passed parameters are uncorrect.")

    def get_movement(self, value="*", causal="*", notes="*", direction="*",
        day="*", month="*", year="*"):
        """
        this function returns a list of movements with the given parameters.
        if parameters are uncorrect, it will return a void list.
        """
        return self.model.get_movement(value, causal, notes, direction, year, month, day)

    def get_movement_by_id(self, ID):
        """
        this function returns the movement with the given id, None if id doesn't
         exists.
        """
        return self.model.get_movement_by_id(ID)

    def remove_movement(self, ID):
        """
        this function removes the movement with the given id.
        if id doesn't exist, it will raise a ValueError
        Not written yet.
        """
        pass

    def get_last_movement(self):
        """
        this function return the last movement, None if there aren't.
        """
        return self.model.get_last_movement()

if __name__=="__main__":
    """ we'll run unittest.... """
    import unittest

    class Test_Controller(unittest.TestCase):
        def setUp(self):
            self.contr=Controller()
            self.movements=[[33.50, u'Spese extra', u'Wired', u'Expense', '2','5','2010'],
            [5, u'Spese extra', u'pantalone', u'Expense', '12','12','1999'],
            [10, u'Altre entrate', u'rimborso benzina', u'Income', '15','8','2009'],
            [1000, u'Stipendio', u'stipendio', u'Income', '2', '5', '2010'],
            ]

        def test_is_date_correct(self):
            #correct dates: correct strings
            self.assertEqual(self.contr.is_date_correct("20/10/2010"), True)
            self.assertEqual(self.contr.is_date_correct("20/01/2010"), True)
            self.assertEqual(self.contr.is_date_correct("20/1/2010"), True)
            self.assertEqual(self.contr.is_date_correct("2/1/2010"), True)
            self.assertEqual(self.contr.is_date_correct("2/01/2010"), True)
            #correct dates: passed a datetime.date object
            self.assertEqual(self.contr.is_date_correct(datetime.date.today()), True)
            #uncorrect date: the year is not in range 1970-2038
            self.assertEqual(self.contr.is_date_correct("20/01/020"), False)
            #uncorrect date: it is not a date!
            self.assertEqual(self.contr.is_date_correct("12/12/1993/33/55"), False)
            self.assertEqual(self.contr.is_date_correct("12/12/1993/aa/bb"), False)
            #uncorrect date: the date is in the future! (we write at 2010...
            #                                        update it if necessary!)
            self.assertEqual(self.contr.is_date_correct("2/10/2037"), False)
            #uncorrect date: the passed string doesn't match with the regex
            self.assertEqual(self.contr.is_date_correct("1234567890"), False)
            #uncorrect date: it's an uncorrect object!
            self.assertEqual(self.contr.is_date_correct(xrange(12)), False)

        def test_control_direction(self):
            self.assertEqual(self.contr.control_direction(self.movements[0][3]), True)
            self.assertEqual(self.contr.control_direction(self.movements[1][3]), True)
            self.assertEqual(self.contr.control_direction(self.movements[2][3]), True)
            self.assertEqual(self.contr.control_direction(self.movements[3][3]), True)

        def test_is_causal_correct(self):
            self.assertEqual(self.contr.is_causal_correct(self.movements[0][1]), True)
            self.assertEqual(self.contr.is_causal_correct(self.movements[1][1]), True)
            self.assertEqual(self.contr.is_causal_correct(self.movements[2][1]), True)
            self.assertEqual(self.contr.is_causal_correct(self.movements[3][1]), True)

        def test_is_movement_correct(self):
            mov=self.movements
            self.assertEqual(self.contr.is_movement_correct(mov[0][0], mov[0][1],
                    mov[0][2], mov[0][3], "/".join(mov[0][4:])), True)
            self.assertEqual(self.contr.is_movement_correct(mov[1][0], mov[1][1],
                    mov[1][2], mov[1][3], "/".join(mov[1][4:])), True)
            self.assertEqual(self.contr.is_movement_correct(mov[2][0], mov[2][1],
                    mov[2][2], mov[2][3], "/".join(mov[2][4:])), True)
            self.assertEqual(self.contr.is_movement_correct(mov[3][0], mov[3][1],
                    mov[3][2], mov[3][3], "/".join(mov[3][4:])), True)
                    
        def test_update_movement(self):
            mov=self.movements
            #self.assertEqual(self.contr.updateMovement

    unittest.main()
